@@ -60,6 +60,9 @@ class OrmBoundField:
     def parse_lookup(self, lookup, value):
         return self.type_.parse_lookup(lookup, value, self.choices)
 
+    def format_lookup(self, lookup, value):
+        return self.type_.format_lookup(lookup, value, self.choices)
+
     @classmethod
     def blank(cls):
         return cls(
@@ -179,7 +182,7 @@ class OrmCalculatedField(OrmBaseField):
             previous=previous,
             full_path=previous.full_path + [self.name],
             pretty_path=previous.pretty_path + [self.pretty_name],
-            queryset_path=previous.queryset_path + ["id"],
+            queryset_path=previous.queryset_path + ["pk"],
             model_name=self.model_name,
         )
 
@@ -208,7 +211,7 @@ class OrmBoundAnnotatedField(OrmBoundField):
             **{
                 s(self.queryset_path): Subquery(
                     admin_get_queryset(self.admin, request, [self.name])
-                    .filter(pk=OuterRef(s(self.previous.queryset_path + ["id"])))
+                    .filter(pk=OuterRef(s(self.previous.queryset_path + ["pk"])))
                     .values(self.name)[:1],
                     output_field=self.django_field,
                 )
@@ -262,7 +265,7 @@ class OrmFileField(OrmConcreteField):
     def get_formatter(self):
         def format(value):
             if not value:
-                return None
+                return value
 
             try:
                 # some storage backends will hard fail if their underlying storage isn't
