@@ -1,7 +1,6 @@
 import cProfile
 import csv
 import io
-import itertools
 import json
 import marshal
 import pstats
@@ -21,6 +20,7 @@ from django.views.decorators import csrf
 
 from . import version
 from .common import HttpResponse, JsonResponse, can_make_public, settings
+from .format_csv import get_csv_rows
 from .models import View
 from .orm_admin import OPEN_IN_ADMIN, get_models
 from .orm_results import get_result_queryset, get_results
@@ -143,19 +143,19 @@ def query_ctx(request, *, model_name="", fields=""):
 
 
 def admin_action(request, model_name, fields):
-    action = request.POST["action"]
-    field = request.POST["field"]
+    data = json.loads(request.body)
+    action = data["action"]
+    field = data["field"]
 
-    assert field in fields
-    if field not in fields.split(","):
-        return  # pragma: no cover
+    if field not in fields:
+        raise http.Http404(f"bad field '{field}'")  # pragma: no cover
 
     params = hyperlink.parse(request.get_full_path()).query
     query = Query.from_request(model_name, field, params)
 
     orm_models = get_models(request)
     if query.model_name not in orm_models:
-        raise http.Http404(f"{query.model_name} does not exist")  # pragma: no cover
+        raise http.Http404(f"'{query.model_name}' does not exist")  # pragma: no cover
 
     bound_query = BoundQuery.bind(query, orm_models)
 
@@ -167,7 +167,7 @@ def admin_action(request, model_name, fields):
     pks -= {None}
 
     model_name = bound_query.fields[0].orm_bound_field.field.model_name
-    return orm_models[model_name].do_action(request, action, pks)
+    return orm_models[model_name].get_action_request(request, action, pks)
 
 
 @csrf.csrf_protect
@@ -175,9 +175,7 @@ def admin_action(request, model_name, fields):
 @admin_decorators.staff_member_required
 def query_html(request, *, model_name="", fields=""):
     if request.method == "POST":
-        resp = admin_action(request, model_name, fields)
-        if resp:  # pragma: no branch
-            return resp
+        return admin_action(request, model_name, fields)
 
     config = _get_config(request)
     config = json.dumps(config, cls=DjangoJSONEncoder)
@@ -261,38 +259,6 @@ def view(request, pk, media):
         raise http.Http404("No View matches the given query.")
 
 
-def pad(x):
-    return [None] * max(0, x)
-
-
-def concat(*lists):
-    return itertools.chain.from_iterable(lists)
-
-
-def flip_table(table):
-    return (list(x) for x in zip(*table))
-
-
-def join_tables(*tables):
-    return (list(concat(*ts)) for ts in zip(*tables))
-
-
-def format_table(fields, table, spacing=0):
-    return concat(
-        [[" ".join(f.pretty_path) for f in fields]],
-        *[
-            [[(row[f.path_str] if row else "") for f in fields]]
-            + [pad(len(fields))] * spacing
-            for row in table
-        ],
-    )
-
-
-def pad_table(x, table):
-    p = pad(x)
-    return (p + row for row in table)
-
-
 class Echo:
     def write(self, value):
         return value
@@ -309,35 +275,11 @@ def _data_response(request, query, media, privileged=False, strict=False):
 
     if media == "csv":
         results = get_results(request, bound_query, orm_models, False)
-
-        def csv_rows():
-            # the pivoted column headers
-            yield from pad_table(
-                len(bound_query.row_fields) - 1,
-                flip_table(
-                    format_table(
-                        bound_query.col_fields,
-                        results["cols"],
-                        spacing=len(bound_query.data_fields) - 1,
-                    )
-                ),
-            )
-
-            # the row headers and data area
-            yield from pad_table(
-                1 - len(bound_query.row_fields),
-                join_tables(
-                    format_table(bound_query.row_fields, results["rows"]),
-                    *(
-                        format_table(bound_query.data_fields, sub_table)
-                        for sub_table in results["body"]
-                    ),
-                ),
-            )
+        csv_rows = get_csv_rows(bound_query, results)
 
         writer = csv.writer(Echo())
         response = http.StreamingHttpResponse(
-            (writer.writerow(row) for row in csv_rows()), content_type="text/csv"
+            (writer.writerow(row) for row in csv_rows), content_type="text/csv"
         )
         response[
             "Content-Disposition"
