@@ -1,6 +1,6 @@
 import React, { useContext } from "react";
 import "./App.css";
-import { TLink, SLink, Overlay, syncPost } from "./Util";
+import { TLink, SLink, Overlay, syncPost, doPost } from "./Util";
 import { ShowContextMenu } from "./ContextMenu";
 
 function Spacer(props) {
@@ -14,9 +14,17 @@ function Spacer(props) {
 }
 
 function HeadCell(props) {
-  const { query, field, className } = props;
+  const {
+    query,
+    field,
+    className,
+    leftArrow,
+    rightArrow,
+    verticalArrows,
+  } = props;
   const modelField = query.getField(field.pathStr);
   const type = query.getType(modelField);
+  const fieldClass = query.getFieldClass(modelField);
 
   const showContextMenu = useContext(ShowContextMenu);
 
@@ -26,17 +34,32 @@ function HeadCell(props) {
       modelField.actions.map((action) => {
         return {
           name: action.prettyName,
-          fn: () => syncPost("", { action: action.name, field: field.pathStr }),
+          fn: () =>
+            doPost("", {
+              action: action.name,
+              field: field.pathStr,
+            }).then((response) => syncPost(response.url, response.data)),
         };
       })
     );
   }
 
   return (
-    <th className={`HeadCell ${className}`} onContextMenu={onContextMenu}>
+    <th
+      className={`HeadCell ContextCursor ${className} ${fieldClass}`}
+      onContextMenu={onContextMenu}
+    >
       <SLink onClick={() => query.removeField(field)}>close</SLink>
-      <SLink onClick={() => query.moveField(field, true)}>chevron_left</SLink>
-      <SLink onClick={() => query.moveField(field, false)}>chevron_right</SLink>
+      {leftArrow && (
+        <SLink onClick={() => query.moveField(field, true)}>
+          {verticalArrows ? "expand_less" : "chevron_left"}
+        </SLink>
+      )}
+      {rightArrow && (
+        <SLink onClick={() => query.moveField(field, false)}>
+          {verticalArrows ? "expand_more" : "chevron_right"}
+        </SLink>
+      )}
       {modelField.canPivot && (
         <>
           <SLink onClick={() => query.togglePivot(field)}>
@@ -134,7 +157,7 @@ function DataCell(props) {
 
   return (
     <td
-      className={`DataCell ${modelField.type + " " + className || ""}`}
+      className={`DataCell ContextCursor ${modelField.type} ${className}`}
       colSpan={span || 1}
       onContextMenu={onContextMenu}
     >
@@ -150,6 +173,9 @@ function VTableHeadRow(props) {
       {...{ query, field }}
       key={field.pathStr}
       className={`HoriBorder ${className} ` + (i ? "" : classNameFirst)}
+      verticalArrows={false}
+      leftArrow={i !== 0}
+      rightArrow={i !== fields.length - 1}
     />
   ));
 }
@@ -189,10 +215,22 @@ function VTableBodyRow(props) {
 }
 
 function HTableRow(props) {
-  const { query, field, data, span, className, formatHints } = props;
+  const {
+    query,
+    field,
+    data,
+    span,
+    className,
+    formatHints,
+    leftArrow,
+    rightArrow,
+  } = props;
   return (
     <>
-      <HeadCell {...{ query, field }} />
+      <HeadCell
+        {...{ query, field, leftArrow, rightArrow }}
+        verticalArrows={true}
+      />
       {data.map((col, key) => (
         <DataCell
           {...{ key, span, className, query }}
@@ -209,67 +247,82 @@ function HTableRow(props) {
 
 function Results(props) {
   const { query, cols, rows, body, overlay, formatHints } = props;
+  const colFields = query.colFields();
+  const topTitleSpace = query.rowFields().length - 1;
+  const sideTitleSpace = query.colFields().length
+    ? 1 - query.rowFields().length
+    : 0;
+  const hasBody = query.rowFields().length || query.bodyFields().length || null;
+
   return (
     <div className="Results">
       <Overlay message={overlay} />
       <div className="Scroller">
         <table>
           <thead>
-            {/* pivoted data */}
-            {query.colFields().map((field) => {
+            {/* col headers and data aka pivots */}
+            {colFields.map((field, i) => {
               return (
                 <tr key={field.pathStr}>
-                  <Spacer spaces={query.rowFields().length - 1} />
+                  <Spacer spaces={topTitleSpace} />
                   <HTableRow
                     {...{ query, field, formatHints }}
-                    span={query.resFields().length}
+                    span={query.bodyFields().length}
                     data={cols}
                     className={overlay && "Fade"}
+                    leftArrow={i !== 0}
+                    rightArrow={i !== colFields.length - 1}
                   />
                 </tr>
               );
             })}
 
-            {/* column headers */}
-            <tr>
-              <Spacer spaces={1 - query.rowFields().length} />
-              <VTableHeadRow
-                {...{ query }}
-                fields={query.rowFields()}
-                className="Freeze"
-              />
-              {cols.map((_, key) => (
+            {/* body/aggregate headers */}
+            {hasBody && (
+              <tr>
+                <Spacer spaces={sideTitleSpace} />
                 <VTableHeadRow
-                  {...{ key, query }}
-                  fields={query.resFields()}
-                  classNameFirst="LeftBorder"
+                  {...{ query }}
+                  fields={query.rowFields()}
                   className="Freeze"
                 />
-              ))}
-            </tr>
+                {(cols.length ? cols : [null]).map((_, key) => (
+                  <VTableHeadRow
+                    {...{ key, query }}
+                    fields={query.bodyFields()}
+                    classNameFirst="LeftBorder"
+                    className="Freeze"
+                  />
+                ))}
+              </tr>
+            )}
           </thead>
 
           {/* row headers and body */}
           <tbody className={overlay && "Fade"}>
-            {rows.map((row, rowIndex) => (
-              <tr key={rowIndex}>
-                <Spacer spaces={1 - query.rowFields().length} />
-                <VTableBodyRow
-                  {...{ query, row, formatHints }}
-                  fields={query.rowFields()}
-                  fullRow={row}
-                />
-                {body.map((table, key) => (
-                  <VTableBodyRow
-                    {...{ key, query, formatHints }}
-                    fields={query.resFields()}
-                    row={table[rowIndex]}
-                    fullRow={{ ...row, ...cols[key] }}
-                    classNameFirst="LeftBorder"
-                  />
-                ))}
-              </tr>
-            ))}
+            {hasBody &&
+              rows.map(
+                (row, rowIndex) =>
+                  row && (
+                    <tr key={rowIndex}>
+                      <Spacer spaces={sideTitleSpace} />
+                      <VTableBodyRow
+                        {...{ query, row, formatHints }}
+                        fields={query.rowFields()}
+                        fullRow={row}
+                      />
+                      {body.map((table, key) => (
+                        <VTableBodyRow
+                          {...{ key, query, formatHints }}
+                          fields={query.bodyFields()}
+                          row={table[rowIndex]}
+                          fullRow={{ ...row, ...cols[key] }}
+                          classNameFirst="LeftBorder"
+                        />
+                      ))}
+                    </tr>
+                  )
+              )}
           </tbody>
         </table>
       </div>
